﻿using Ant_Colony_Optimalization.Ant.PheromoneActualization;
using Ant_Colony_Optimalization.Generator;
using Ant_Colony_Optimalization.Graph;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization.Ant
{
    public class CAntShortestWay : IAnt, IComparable
    {
        public int[] VisitedEdgesIndex { get; set; }
        public LinkedList<CEdge> VisitedEdges { get; set; }

        private int stepNumber = 1;
        public LinkedList<CNode> VisitedNodes { get; set; }
        CNode ActualNode { get; set; }
        CNode StartingNode { get; set; }
        CNode EndingNode { get; set; }
        private int visitedNodesCount;
        private int p;
        public bool PathFound { get; set; }

        private double alpha;
        private double beta;
        public double Distance { get; set; }
        private IPheromoneIncrease PheromonIncreaseFunction { get; set; }
        public CAntShortestWay(CNode startingNode, CNode endingNode, int nodeCount,double alpha, double beta, IPheromoneIncrease increaseFunction)
        {
            this.EndingNode = endingNode;
            this.visitedNodesCount = 1;
            this.VisitedEdges = new LinkedList<CEdge>();
            this.VisitedNodes = new LinkedList<CNode>();
            this.VisitedEdgesIndex = new int[nodeCount];
            this.VisitedNodes.AddLast(startingNode);
            this.ActualNode = startingNode;
            this.StartingNode = startingNode;
            this.Distance = 0;
            this.PathFound = false;
            this.PheromonIncreaseFunction = increaseFunction;
        }

        public bool MoveToNextNode()
        {

            Tuple<int, CEdge>[] notVisitedEdgesArray = this.GetEdgesToNotVisitedNodes();
            if (notVisitedEdgesArray.Count() == 0)
            {
                this.PathFound = false;
                return false;
            }
            else
            {
                int edgeIndex = this.ChooseNextNodeIndex2(notVisitedEdgesArray);
                CEdge choosenEdge = this.GetEdgeByIndex(edgeIndex);
                this.Distance += choosenEdge.Distance;
                CNode choosenNode = this.GetNeighboringNodeOnEdge(choosenEdge);
                this.VisitedEdges.AddLast(choosenEdge);
                this.VisitedEdgesIndex[this.stepNumber - 1] = edgeIndex;
                this.VisitedNodes.AddLast(choosenNode);
                this.stepNumber++;
                this.ActualNode = choosenNode;
                this.visitedNodesCount++;
                if (choosenNode.Equals(this.EndingNode))
                {
                    this.PathFound = true;
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }

        private int ChooseNextNodeIndex()
        {
            double[] probabilityArray = this.ActualNode.CumulProbabilityArray;
            double randomNumber = CRandomProvider.GetThreadRandom().NextDouble();
            int counter = 0;
            //pokud by náhodné číslo bylo 1.0 rozhodilo by to while cyklus
            if (randomNumber == 1.0)
            {
                counter = probabilityArray.Count() - 1;
            }
            while (randomNumber > probabilityArray[counter])
            {
                counter++;
            }
            return counter;
        }

        private Tuple<int, CEdge>[] GetEdgesToNotVisitedNodes()
        {
            CEdge[] nodeEdgesArray = this.ActualNode.EdgeArray;
            LinkedList<Tuple<int, CEdge>> notVisitedEdgesList = new LinkedList<Tuple<int, CEdge>>();

            for (int a = 0; a < nodeEdgesArray.Count(); a++)
            {
                CNode nodeNeighbor = nodeEdgesArray[a].GetNeighbor(this.ActualNode);
                if (this.VisitedNodes.Contains(nodeNeighbor))
                {

                }
                else
                {
                    notVisitedEdgesList.AddLast(new Tuple<int, CEdge>(a, nodeEdgesArray[a]));
                }
            }
            return notVisitedEdgesList.ToArray();
        }

        private int ChooseNextNodeIndex2(Tuple<int, CEdge>[] notVisitedEdgesArray)
        {
            if (notVisitedEdgesArray.Count() == 1)
            {
                return notVisitedEdgesArray[0].Item1;
            }
            else
            {
                double[] probabilityArray = this.RecalculateProbabilityArray(notVisitedEdgesArray);
                double[] cumProbabilityArray = this.RecalculateCumulProbabilityArray(probabilityArray);

                double randomNumber = this.GetRandomNumber(0, 1);
                int counter = 0;

                while (randomNumber >= cumProbabilityArray[counter])
                {
                    counter++;
                }
                return notVisitedEdgesArray[counter].Item1;
            }
        }

        private double[] RecalculateProbabilityArray(Tuple<int, CEdge>[] notVisitedEdgesArray)
        {
            LinkedList<double> probabilityList = new LinkedList<double>();
            // double[] probabilityArray = new double[notVisitedEdgesArray.Count()];

            for (int i = 0; i < notVisitedEdgesArray.Count(); i++)
            {
                CEdge edge = notVisitedEdgesArray[i].Item2;
                probabilityList.AddLast(this.CalculateTauete2(edge.Pheromone, edge.Distance));
            }
            return probabilityList.ToArray();
        }
        public double[] RecalculateCumulProbabilityArray(double[] probabilityArray)
        {
            double[] cumulProbabilityArray = new double[probabilityArray.Count()];
            double arraySum = probabilityArray.Sum();


            cumulProbabilityArray[0] = probabilityArray[0] / arraySum;
            for (int i = 1; i < probabilityArray.Count(); i++)
            {
                double i1 = cumulProbabilityArray[i - 1];
                double i2 = probabilityArray[i] / arraySum;
                cumulProbabilityArray[i] = i1 + i2;
            }
            cumulProbabilityArray[cumulProbabilityArray.Count() - 1] = 1;

            return cumulProbabilityArray;
        }

        private CEdge GetEdgeByIndex(int index)
        {
            return this.ActualNode.EdgeArray[index];
        }

        private CNode GetNeighboringNodeOnEdge(CEdge edge)
        {
            if (edge.Node1 == (this.ActualNode))
            {
                return edge.Node2;
            }
            else
            {
                return edge.Node1;
            }
        }

        public double CalculateTauete(double pheromone, double distance)
        {
            double tauete = System.Math.Pow(pheromone, this.alpha) * System.Math.Pow((1 / distance), this.beta);
            return tauete;
        }

        public double CalculateTauete2(double pheromone, double distance)
        {
            double tauete = pheromone;
            return tauete;
        }

        public void ActualizePheromone(double pheromoneIncrease)
        {
            foreach (CEdge edge in this.VisitedEdges)
            {
                if (edge != null)
                {
                    if (edge.Pheromone < 5)
                    {
                        if (edge.Pheromone < 0.2)
                        {
                            edge.Pheromone = edge.Pheromone * pheromoneIncrease * 3;
                        }
                        else
                        {
                            edge.Pheromone = edge.Pheromone * pheromoneIncrease;
                        }
                    }
                }
            }
        }

        private double GetRandomNumber(double minimum, double maximum)
        {
            Random random = CRandomProvider.GetThreadRandom();
            return (random.NextDouble() * (maximum - minimum) + minimum);
        }

        public int CompareTo(object obj)
        {
            return this.Distance.CompareTo(((CAntShortestWay)obj).Distance);
        }


        public void ActualizePheromone()
        {
            this.PheromonIncreaseFunction.IncreasePheromone(this.VisitedEdges.ToArray());
        }
    }
}
