﻿using Ant_Colony_Optimalization.Graph;
using Ant_Colony_Optimalization.Graphic.Object;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization.Generator
{
    public class CNodeGenerator
    {
        int height;
        int width;
        public CNodeGenerator(int height, int width)
        {
            this.height = height;
            this.width = width;
        }

        public CNode[] GetRandom(int count)
        {
            int counter = 0;
            LinkedList<CNode> nodeList = new LinkedList<CNode>();

            while (counter < count)
            {
                Random random = new Random();
                int x = this.GetRandomNumber(10, this.width - 10);
                int y = this.GetRandomNumber(10, this.height - 10);
                CNodePoint p = new CNodePoint(x, y);
                CNode newNode = new CNode(counter, p, 0);
                if (nodeList.Contains(newNode))
                {
                    // MessageBox.Show("Stejné");
                }
                else
                {
                    nodeList.AddLast(newNode);
                    counter++;
                }
            }
            return nodeList.ToArray();
        }

        public CNode[] GetSiteList(int rowsCount, int columnsCount)
        {
            int counter = 0;
            LinkedList<CNode> nodeList = new LinkedList<CNode>();
            int rozestupRows = this.width / (columnsCount + 1);
            int rozestupColumns = this.height / (rowsCount + 1);

            int rozestup;

            if (rozestupRows < rozestupColumns)
            {
                rozestup = rozestupRows;
            }
            else
            {
                rozestup = rozestupColumns;
            }

            for (int y = 1; y <= rowsCount; y++)
            {

                for (int x = 1; x <= columnsCount; x++)
                {

                    CNodePoint p = new CNodePoint(x * rozestup, y * rozestup);
                    CNode newNode = new CNode(nodeList.Count, p, 0);
                    nodeList.AddLast(newNode);
                    counter++;
                }
            }
            return nodeList.ToArray();
        }

        public CNode[,] GetSiteArray(int rowsCount, int columnsCount)
        {
            int counter = 0;
            CNode[,] nodeArray = new CNode[columnsCount, rowsCount];
            int rozestupRows = this.width / (columnsCount + 1);
            int rozestupColumns = this.height / (rowsCount + 1);

            for (int y = 1; y <= rowsCount; y++)
            {

                for (int x = 1; x <= columnsCount; x++)
                {

                    CNodePoint p = new CNodePoint(x * rozestupRows, y * rozestupColumns);
                    CNode newNode = new CNode(counter, p, 0);
                    nodeArray[x - 1, y - 1] = newNode;
                    counter++;
                }
            }
            return nodeArray;
        }

        public CNode[] GetCircle(int size, int count)
        {
            int SIZE = size ;
            int a = SIZE / 2;
            int b = a;
            int r = 4 * SIZE / 5;
            int n;

            LinkedList<CNode> nodeList = new LinkedList<CNode>();


            int m = Math.Min(a, b);
            r = 4 * m / 5;
            int r2 = Math.Abs(m - r) / 2;

            for (int i = 0; i < count; i++)
            {
                double t = 2 * Math.PI * i / count;
                int x = (int)Math.Round(a + r * Math.Cos(t));
                int y = (int)Math.Round(b + r * Math.Sin(t));
                CNodePoint p = new CNodePoint(x, y);
                CNode newNode = new CNode(nodeList.Count, p, 0);
                nodeList.AddLast(newNode);
            }

            return nodeList.ToArray();
        }
        /*
                public LinkedList<CNode> GetCircles(int size, int size2)
                {
                    LinkedList<CNode> nodeList1 =this.GetCircle(size);
                    foreach (CNode node in this.GetCircle(size2))
                    {
                        nodeList1.AddLast(node);
                    }
                    return nodeList1;
                }
        */
        private int GetRandomNumber(double minimum, double maximum)
        {
            Random random = CRandomProvider.GetThreadRandom();
            return Convert.ToInt32(random.NextDouble() * (maximum - minimum) + minimum);
        }
    }
}
