﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Ant_Colony_Optimalization.Graphic.Object;
using Ant_Colony_Optimalization.Graphic.Render;
using Ant_Colony_Optimalization.Graph;
using Ant_Colony_Optimalization.Enum;

namespace Ant_Colony_Optimalization.UserControls
{
    public partial class UserControlGraph : UserControl
    {
        private FormMain mainForm;
        private CNodeRender nodeRender;
        private CEdgeRender edgerender;
        private ERenderMode renderMode;


        public CGraphicObjectHolder GraphicHolder { get; set; }
        public int PanelHeight
        {
            get { return this.panel1.Height; }
        }

        public int PanelWidth
        {
            get { return this.panel1.Width; }
        }

        public UserControlGraph()
        {
            InitializeComponent();
        }
        private void UserControlGraph_Load(object sender, EventArgs e)
        {
            this.nodeRender = new CNodeRender(this.panel1);
            this.edgerender = new CEdgeRender(this.panel1);
            this.GraphicHolder = new CGraphicObjectHolder();
            this.mainForm = (FormMain)this.Parent.Parent;
        }

        public void RenderGraphic(ERenderMode mode)
        {
            this.renderMode = mode;
            this.panel1.Invalidate();
        }

        public void ClearPanel()
        {
            this.panel1.Invalidate();
        }

        private void panel1_MouseMove(object sender, MouseEventArgs e)
        {
            this.labelX.Text = "X : " + e.X.ToString();
            this.labelY.Text = "Y : " + e.Y.ToString();
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            /*
            if (this.GraphicHolder.NodeHighlighted)
            {
                this.edgerender.DrawPink(this.GraphicHolder.PinkEdges);
            }*/

            if (renderMode == ERenderMode.EdgeHighlighted)
            {
                this.edgerender.DrawPink(this.GraphicHolder.PinkEdges);
                this.edgerender.DrawGreen(this.GraphicHolder.ResultEdges);
                this.nodeRender.Draw(this.GraphicHolder.NormalNodes);
            }
            else if (this.renderMode == ERenderMode.Graph)
            {
                if (this.GraphicHolder.NormalNodes.Count() <= 100) { 
                    this.edgerender.DrawAll(this.GraphicHolder.AllEdges);
                }
                this.nodeRender.Draw(this.GraphicHolder.NormalNodes);
            }
            else if (renderMode == ERenderMode.Result)
            {
                this.edgerender.DrawGreen(this.GraphicHolder.ResultEdges);
                this.nodeRender.Draw(this.GraphicHolder.NormalNodes);
            }
            else if (renderMode == ERenderMode.AntWay)
            {
                this.edgerender.DrawGreen(this.GraphicHolder.ResultEdges);
                this.nodeRender.Draw(this.GraphicHolder.NormalNodes);
                this.edgerender.DrawBrown(this.GraphicHolder.AntWay);
            }
            else if (renderMode == ERenderMode.Pheromone)
            {
                int pheromoneWidth = this.panel1.Width / (this.GraphicHolder.PathColumnCount + 1);
                int pheromoneHeight = this.panel1.Height / (this.GraphicHolder.PathRowCount + 1);

                this.nodeRender.DrawPheromone(this.GraphicHolder.NodePheromoneStrenge, pheromoneWidth, pheromoneHeight);
                //this.edgerender.DrawPink(this.GraphicHolder.PreviousResultEdges);
                this.edgerender.DrawRed(this.GraphicHolder.ResultEdges);
               
                //this.edgerender.DrawAll(this.GraphicHolder.AllEdges);
                //this.nodeRender.Draw(this.GraphicHolder.NormalNodes);
            }
            else
            {
                this.edgerender.DrawAll(this.GraphicHolder.AllEdges);
                this.nodeRender.Draw(this.GraphicHolder.NormalNodes);
                this.edgerender.DrawGreen(this.GraphicHolder.ResultEdges);
            }
        }

        private void panel1_MouseClick(object sender, MouseEventArgs e)
        {
            CNodePoint p = new CNodePoint(e.X, e.Y);
            CNode newNode = new CNode(this.GraphicHolder.NormalNodes.Count(), p, 0);
            if (e.Button == MouseButtons.Left)
            {
                if (this.mainForm.GetLeftPanelType() == typeof(UserControlTSPResults) || this.mainForm.GetLeftPanelType() == typeof(UserControlPathResult))
                {
                    if (this.GraphicHolder.NormalNodes.ToList().Contains(newNode))
                    {
                        if (this.GraphicHolder.NodeHighlighted)
                        {
                            this.GraphicHolder.NodeHighlighted = false;
                            this.GraphicHolder.PinkEdges = new LinkedList<CEdgeLine>();
                        }
                        else
                        {
                            List<CNode> nodeList = this.GraphicHolder.NormalNodes.ToList();
                            CNode originalNode = this.GraphicHolder.NormalNodes[Array.IndexOf(this.GraphicHolder.NormalNodes, newNode)];
                            // this.GraphicHolder.NodeHighlighted = true;
                            this.GraphicHolder.PinkEdges = new LinkedList<CEdgeLine>();
                            for (int i = 0; i < originalNode.EdgeArray.Count(); i++)
                            {
                                this.GraphicHolder.PinkEdges.AddLast(originalNode.EdgeArray[i].ToCEdgeLine());
                            }
                            this.RenderGraphic(ERenderMode.Result);

                            if (this.mainForm.GetLeftPanelType() == typeof(UserControlTSPResults))
                            {
                                UserControlTSPResults ucRes = (UserControlTSPResults)this.mainForm.GetLeftPanel();
                                ucRes.ShowNodeInfo(originalNode);
                            }
                            else
                            {
                                UserControlPathResult ucRes = (UserControlPathResult)this.mainForm.GetLeftPanel();
                                ucRes.ShowNodeInfo(originalNode);
                            }
                        }
                    }
                }
                else
                {
                    if (this.GraphicHolder.NormalNodes.ToList().Contains(newNode))
                    {
                        // MessageBox.Show("Stejné");
                    }
                    else
                    {
                        List<CNode> nodeList = this.GraphicHolder.NormalNodes.ToList();
                        nodeList.Add(newNode);
                        this.GraphicHolder.NormalNodes = nodeList.ToArray();

                        this.panel1.Invalidate();
                    }
                }
            }
        }


    }
}
