﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Ant_Colony_Optimalization.Enum;
using Ant_Colony_Optimalization.Ant.PheromoneActualization;
using Ant_Colony_Optimalization.Other;
using Ant_Colony_Optimalization.Graph;
using Ant_Colony_Optimalization.Graphic.Object;
using Ant_Colony_Optimalization.Generator;

namespace Ant_Colony_Optimalization.UserControls
{
    public partial class UserControlPathPanel : UserControl
    {
        private FormMain mainForm;
        private UserControlGraph graphPanel;
        private CNode[,] nodeArray;
        public UserControlPathPanel()
        {
            InitializeComponent();
        }

        private void buttonBackToMainMenu_Click(object sender, EventArgs e)
        {
            this.mainForm.ShowMainMenu();
        }

        private void UserControlPathPanel_Load(object sender, EventArgs e)
        {
            this.mainForm = (FormMain)this.Parent.Parent;
            this.graphPanel = this.mainForm.GraphPanelUserControl;
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            if (this.graphPanel.GraphicHolder.AllEdges.Length ==0)
            {
                //CNode[,] nodeArray = new CNodeGenerator(this.graphPanel.PanelHeight, this.graphPanel.PanelWidth).GetSiteArray(Convert.ToInt32(this.textBoxRowCount.Text), Convert.ToInt32(this.textBoxColumnCount.Text));
                this.graphPanel.GraphicHolder.AllEdges = new CEdgeGenerator().CalculateEdges(this.nodeArray).ToArray();
            }
            this.graphPanel.RenderGraphic(ERenderMode.Graph);

            IPheromoneIncrease increaseFunction = null;
            if (radioButtonIConstant.Checked)
            {
                increaseFunction = new CIncreaseConstant(Convert.ToDouble(this.numericUpDownIPheromone.Value));
            }
            else if (radioButtonIMultiplication.Checked)
            {
                increaseFunction = new CIncreaseMultiply(Convert.ToDouble(this.numericUpDownIPheromone.Value), Convert.ToDouble(this.numericUpDownMax.Value));
            }
            else if (radioButtonIPlus.Checked)
            {
                increaseFunction = new CIncreasePlus(Convert.ToDouble(this.numericUpDownIPheromone.Value), Convert.ToDouble(this.numericUpDownMax.Value));
            }

            IPheromoneDecrease decreaseFunction = null;
            if (radioButtonDConstant.Checked)
            {
                decreaseFunction = new CDecreaseConstant(Convert.ToDouble(this.numericUpDownDPheromone.Value));
            }
            else if (radioButtonDMultiplication.Checked)
            {
                decreaseFunction = new CDecreaseMultiply(Convert.ToDouble(this.numericUpDownDPheromone.Value), Convert.ToDouble(this.numericUpDownMin.Value));
            }
            else if (radioButtonDMinus.Checked)
            {
                decreaseFunction = new CDecreaseMinus(Convert.ToDouble(this.numericUpDownDPheromone.Value), Convert.ToDouble(this.numericUpDownMin.Value));
            }

            double betterThan;
            if (this.checkBoxBetterThan.Checked){
                 betterThan= Convert.ToDouble(this.numericUpDownBetterThan.Value);
            } else{
                 betterThan = 0;
            }
            this.graphPanel.GraphicHolder.PathColumnCount= Convert.ToInt32(this.numericUpDownColumnCount.Value);
            this.graphPanel.GraphicHolder.PathRowCount= Convert.ToInt32(this.numericUpDownRowCount.Value);
            CSettingsHolder settings = new CSettingsHolder(Convert.ToDouble(this.numericUpDownAlfa.Value), Convert.ToDouble(this.numericUpDownBeta.Value), 12, Convert.ToInt32(this.numericUpDownBestCount.Value), betterThan, increaseFunction, decreaseFunction, Convert.ToInt32(this.numericUpDownStart.Value), Convert.ToInt32(this.numericUpDownFinish.Value));
            this.mainForm.ShowResults(EProblemType.ShortesPath, settings);
        }


        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.mainForm.ShowPanel(EProblemType.ShortesPath);
        }

        private void buttonGenerate_Click(object sender, EventArgs e)
        {
                this.graphPanel.GraphicHolder = new CGraphicObjectHolder();

                this.nodeArray = new CNodeGenerator(this.graphPanel.PanelHeight, this.graphPanel.PanelWidth).GetSiteArray(Convert.ToInt32(this.numericUpDownColumnCount.Text), Convert.ToInt32(this.numericUpDownRowCount.Text));
                 LinkedList<CNode> nodeList = new LinkedList<CNode>();
                foreach (CNode node in nodeArray)
                {
                    nodeList.AddLast(node);
                }

                this.graphPanel.GraphicHolder.NormalNodes = nodeList.ToArray();
                this.graphPanel.RenderGraphic(ERenderMode.Graph);
                this.buttonRun.Enabled = true;
                this.numericUpDownStart.Enabled = true;
                this.numericUpDownFinish.Enabled = true;
                this.numericUpDownFinish.Maximum = this.graphPanel.GraphicHolder.NormalNodes.Length - 1;
                this.numericUpDownFinish.Value = this.graphPanel.GraphicHolder.NormalNodes.Length - 1;
        }

        private void ButtonClear_Click(object sender, EventArgs e)
        {
            this.graphPanel.GraphicHolder = new CGraphicObjectHolder();
            this.graphPanel.RenderGraphic(ERenderMode.None);
        }
    }
}
