﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Ant_Colony_Optimalization.Other;
using System.Threading;
using Ant_Colony_Optimalization.Graph;
using Ant_Colony_Optimalization.Enum;
using Ant_Colony_Optimalization.Ant;
using Ant_Colony_Optimalization.Graphic.Object;
using Ant_Colony_Optimalization.Generator;
using Ant_Colony_Optimalization.Graphic;

namespace Ant_Colony_Optimalization.UserControls
{
    public partial class UserControlPathResult : UserControl
    {

        private FormMain mainForm;
        private UserControlGraph graphPanel;
        private double bestResult;
        private ManualResetEvent _busy = new ManualResetEvent(false);
        private bool firstRun;
        private CNode[] nodeList;
        private CEdge[] edgeArray;
        private CNode selectedNode;
        private int roundCount;
        private CSettingsHolder runSettings;
        private DateTime startTime;
        private LinkedList<string> logRecordList;
        public UserControlPathResult(CSettingsHolder runSettings)
        {
            InitializeComponent();
            this.bestResult = double.MaxValue;
            this.firstRun = true;
            this.runSettings = runSettings;
            this.logRecordList = new LinkedList<string>();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {

            this.CancelWorker();
            this.mainForm.ShowPanel(EProblemType.ShortesPath);
        }

        private void UserControlResults_Load(object sender, EventArgs e)
        {
            this.mainForm = (FormMain)this.Parent.Parent;
            this.graphPanel = this.mainForm.GraphPanelUserControl;
        }

        private void buttonPause_Click(object sender, EventArgs e)
        {
            this.buttonPause.Enabled = false;
            this.buttonRun.Enabled = true;
            this.buttonNextRound.Enabled = true;
            this.buttonNextStep.Enabled = true;
            this.PauseWorker();
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            //this.graphPanel.GraphicHolder.AllEdges = new CEdge[0];
            this.graphPanel.RenderGraphic(ERenderMode.Pheromone);
            this.buttonPause.Enabled = true;
            this.buttonRun.Enabled = false;
            this.buttonNextRound.Enabled = false;
            this.buttonNextStep.Enabled = false;
            this.startTime = DateTime.Now;
            //this.backgroundWorker3.RunWorkerAsync();
            this.ResumeWorker();
        }

        private void buttonNextStep_Click(object sender, EventArgs e)
        {
            this.backgroundWorker1.RunWorkerAsync();
        }

        private void backgroundWorker3_DoWork(object sender, DoWorkEventArgs e)
        {
            double maxActualPheromone = double.MinValue;
            if (this.firstRun)
            {
                this.nodeList = this.graphPanel.GraphicHolder.NormalNodes;
                this.edgeArray = this.graphPanel.GraphicHolder.AllEdges;
            }
            int nodeListCount = this.nodeList.Count();

            CNode startingNode = nodeList.ElementAt(this.runSettings.StartNodeId);
            CNode endingNode = nodeList.ElementAt(this.runSettings.FinishNodeId);

            List<CAntShortestWay> anti = new List<CAntShortestWay>();

            if (radioButtonAfterCount.Checked)
            {
                int iterationCount = Convert.ToInt32(this.numericUpDownIterationCount.Value) * this.graphPanel.GraphicHolder.NormalNodes.Count();
                this.roundCount = iterationCount;
            }
            if (radioButtonNoImprovement.Checked)
            {
                this.roundCount = int.MaxValue;
            }
            // int kIteration = 0;
            int noChangeFor = 0;

            for (int kIteration = 0; kIteration <= roundCount - 1; kIteration++)
            {
                CAntShortestWay ant = new CAntShortestWay(startingNode, endingNode, this.nodeList.Length, this.runSettings.Aplha, this.runSettings.Beta, this.runSettings.PheromonIncreaseFunction);

                while (ant.MoveToNextNode())
                {

                }

                if (ant.PathFound)
                {
                    anti.Add(ant);
                    if (anti.Count % 100 == 0)
                    {
                        anti.Sort();
                        for (int iu = 0; iu < this.runSettings.BestCount; iu++)
                        {
                            CAntShortestWay ant2 = anti.ElementAt(iu);
                            if (ant2.Distance < (this.bestResult *this.runSettings.BetterThan))
                            {

                                this.runSettings.PheromonDecreaseFunction.DecreasePheromone(edgeArray);

                                ant2.ActualizePheromone(1.35);

                            }
                            if (ant2.Distance < (this.bestResult))
                            {
                                this.graphPanel.GraphicHolder.PreviousResultEdges = new LinkedList<CEdgeLine>(this.graphPanel.GraphicHolder.ResultEdges);
                                this.graphPanel.GraphicHolder.ResultEdges = new LinkedList<CEdgeLine>();
                                for (int i = 0; i < ant2.VisitedNodes.Count() - 1; i++)
                                {
                                    CNode node1 = ant2.VisitedNodes.ElementAt(i);
                                    CNode node2 = ant2.VisitedNodes.ElementAt(i).EdgeArray[ant2.VisitedEdgesIndex.ElementAt(i)].GetNeighbor(ant2.VisitedNodes.ElementAt(i));
                                    this.graphPanel.GraphicHolder.ResultEdges.AddLast(new CEdgeLine(node1.Point, node2.Point));
                                }
                                this.bestResult = ant2.Distance;

                                //
                                List<CNodePhoremoneStrenge> strengeList = new List<CNodePhoremoneStrenge>();
                                foreach (CNode node in this.nodeList)
                                {
                                    strengeList.Add(new CNodeAnalyzator().GetPheremoneStrenge(node));
                                }
                                strengeList.Sort();

                                double max = strengeList.Last().Strenge;
                                if (max > maxActualPheromone)
                                {
                                    maxActualPheromone = max;
                                }
                                foreach (CNodePhoremoneStrenge nodeStrenge in strengeList)
                                {
                                    nodeStrenge.RecalculateColorValue(maxActualPheromone);
                                }

                                this.graphPanel.GraphicHolder.NodePheromoneStrenge = new LinkedList<CNodePhoremoneStrenge>(strengeList);

                                foreach (CEdge edge in edgeArray)
                                {
                                    if (edge.Pheromone > 0.1)
                                    {
                                        edge.Pheromone = edge.Pheromone * 0.95;
                                    }
                                }

                                backgroundWorker3.ReportProgress(kIteration, new ProgressInfo(kIteration, ant.Distance));

                            }
                            if (anti.Count % 100 == 0)
                            {
                                List<CNodePhoremoneStrenge> strengeList = new List<CNodePhoremoneStrenge>();
                                foreach (CNode node in this.nodeList)
                                {
                                    strengeList.Add(new CNodeAnalyzator().GetPheremoneStrenge(node));
                                }
                                strengeList.Sort();

                                double max = strengeList.Last().Strenge;
                                if (max > maxActualPheromone)
                                {
                                    maxActualPheromone = max;
                                }
                                foreach (CNodePhoremoneStrenge nodeStrenge in strengeList)
                                {
                                    nodeStrenge.RecalculateColorValue(max);
                                }

                                this.graphPanel.GraphicHolder.NodePheromoneStrenge = new LinkedList<CNodePhoremoneStrenge>(strengeList);
                                backgroundWorker3.ReportProgress(kIteration, null);
                            }
                        }
                       // anti = new List<CAntShortestWay>(); 
                    }
                    else
                    {

                    }
                }

                backgroundWorker3.ReportProgress(kIteration, null);
                _busy.WaitOne();
                if (this.backgroundWorker3.CancellationPending)
                {
                    kIteration = int.MaxValue - 1;
                }

                if (this.radioButtonNoImprovement.Checked)
                {
                    noChangeFor++;
                    if (noChangeFor == (Convert.ToInt32(this.numericUpDownNoImprovement.Value) * this.graphPanel.GraphicHolder.NormalNodes.Count()))
                    {
                        kIteration = int.MaxValue - 1;
                    }
                }
            }
             

        }

        private void backgroundWorker3_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {

            int nodeCount = this.graphPanel.GraphicHolder.NormalNodes.Count();
            if (e.UserState != null)
            {
                ProgressInfo pi = (ProgressInfo)e.UserState;
                this.labelBestLength.Text = Math.Round(pi.BestDistance).ToString();
                this.labelBestAnt.Text = pi.BestAnt.ToString();
                this.labelIterationValue.Text = (pi.BestAnt / nodeCount).ToString();
                string time = (DateTime.Now - this.startTime).ToString();
                this.labelInTimeValue.Text = time.Substring(0, time.LastIndexOf('.') + 3);
                this.logRecordList.AddLast(pi.ToString());
            }
           // int iteration = Convert.ToInt32(this.labelIterationActualValue.Text) + 1;
            this.labelIterationActualValue.Text = (e.ProgressPercentage / nodeCount).ToString();
            this.labelAntActualValue.Text = (e.ProgressPercentage).ToString();
            if ((e.ProgressPercentage%100 ==0)){
                this.graphPanel.RenderGraphic(ERenderMode.Pheromone);
            }
        }

        private void buttonBackToMainMenu_Click(object sender, EventArgs e)
        {
            this.mainForm.ShowMainMenu();
        }

        public void ShowNodeInfo(CNode node)
        {
            this.selectedNode = node;
            new CNodeAnalyzator(this.dataGridView1, node, this.runSettings.Aplha, this.runSettings.Beta).ShowEdgesInfo();
            this.labelNodeInfo.Text = "Node : " + node.id.ToString();
        }

        public void ResumeWorker()
        {
            // Start the worker if it isn't running
            if (!backgroundWorker3.IsBusy) backgroundWorker3.RunWorkerAsync();
            // Unblock the worker 
            _busy.Set();
        }

        public void PauseWorker()
        {
            // Block the worker
            _busy.Reset();
        }

        public void CancelWorker()
        {
            this.PauseWorker();
            if (backgroundWorker3.IsBusy)
            {
                // Set CancellationPending property to true
                backgroundWorker3.CancelAsync();
                // Unblock worker so it can see that
                _busy.Set();
            }
        }

        private void backgroundWorker3_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            MessageBox.Show("Work finished");
            this.buttonRun.Enabled = true;
            this.buttonNextRound.Enabled = true;
            this.buttonNextStep.Enabled = true;
        }

        private void dataGridView1_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            int edgeId = Convert.ToInt32(dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[0].Value);
            this.graphPanel.GraphicHolder.PinkEdges = new LinkedList<CEdgeLine>();
            this.graphPanel.GraphicHolder.PinkEdges.AddLast(new CEdgeLine(this.selectedNode.EdgeArray[edgeId].Node1.Point, this.selectedNode.EdgeArray[edgeId].Node2.Point));
            this.graphPanel.RenderGraphic(ERenderMode.EdgeHighlighted);
        }

        private void dataGridView1_SelectionChanged(object sender, EventArgs e)
        {
            /* if (this.selectedNode != null)
             {
                 int edgeId = Convert.ToInt32(dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[0].Value);
                 this.graphPanel.GraphicHolder.PinkEdges = new LinkedList<CEdgeLine>();
                 this.graphPanel.GraphicHolder.PinkEdges.AddLast(new CEdgeLine(this.selectedNode.EdgeArray[edgeId].Node1.Point, this.selectedNode.EdgeArray[edgeId].Node2.Point));
                 this.graphPanel.RenderGraphic(ERenderMode.EdgeHighlighted);
             }*/
        }

        private void labelNodeInfo_TextChanged(object sender, EventArgs e)
        {
            this.dataGridView1.Rows[0].Selected = true;
        }

        private void buttonNextRound_Click(object sender, EventArgs e)
        {
            this.roundCount = 1;
            this.ResumeWorker();
            //this.backgroundWorker3.RunWorkerAsync();
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            if (this.firstRun)
            {
                this.nodeList = this.graphPanel.GraphicHolder.NormalNodes;
                this.edgeArray = this.graphPanel.GraphicHolder.AllEdges;
            }


            CNode startingNode = nodeList.ElementAt(this.runSettings.StartNodeId);
            CNode endingNode = nodeList.ElementAt(this.runSettings.FinishNodeId);
            CAntShortestWay ant = new CAntShortestWay(startingNode, endingNode, this.nodeList.Length, this.runSettings.Aplha, this.runSettings.Beta,this.runSettings.PheromonIncreaseFunction);
            while (ant.MoveToNextNode())
            {

            }

            this.graphPanel.GraphicHolder.AntWay = new LinkedList<CEdgeLine>();
            for (int i = 0; i < ant.VisitedNodes.Count() - 1; i++)
            {
                CNode node1 = ant.VisitedNodes.ElementAt(i);
                CNode node2 = ant.VisitedNodes.ElementAt(i).EdgeArray[ant.VisitedEdgesIndex.ElementAt(i)].GetNeighbor(ant.VisitedNodes.ElementAt(i));
                this.graphPanel.GraphicHolder.AntWay.AddLast(new CEdgeLine(node1.Point, node2.Point));
                this.graphPanel.RenderGraphic(ERenderMode.AntWay);
                Thread.Sleep(500);
            }
        }

        private double GetRandomNumber(double minimum, double maximum)
        {
            Random random = CRandomProvider.GetThreadRandom();
            return (random.NextDouble() * (maximum - minimum) + minimum);
        }
    }
}
