﻿using Ant_Colony_Optimalization.Ant.PheromoneActualization;
using Ant_Colony_Optimalization.Enum;
using Ant_Colony_Optimalization.Generator;
using Ant_Colony_Optimalization.Graph;
using Ant_Colony_Optimalization.Graphic.Object;
using Ant_Colony_Optimalization.Other;
using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

namespace Ant_Colony_Optimalization.UserControls
{
    public partial class UserControlTSPPanel : UserControl
    {
        private FormMain mainForm;
        private UserControlGraph graphPanel;
        public UserControlTSPPanel()
        {
            InitializeComponent();
        }

        private void buttonBackToMainMenu_Click(object sender, EventArgs e)
        {
            this.mainForm.ShowMainMenu();
        }

        private void UserControlTSPPanel_Load(object sender, EventArgs e)
        {
            this.mainForm = (FormMain)this.Parent.Parent;
            this.graphPanel = this.mainForm.GraphPanelUserControl;
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            if (this.graphPanel.GraphicHolder.AllEdges.Length == 0)
            {
                this.graphPanel.GraphicHolder.AllEdges = this.CalculateEdges(this.graphPanel.GraphicHolder.NormalNodes);
            }
            this.graphPanel.RenderGraphic(ERenderMode.Graph);

            IPheromoneIncrease increaseFunction = null;
            if (radioButtonIConstant.Checked)
            {
                increaseFunction = new CIncreaseConstant(Convert.ToDouble(this.numericUpDownIPheromone.Value));
            }
            else if (radioButtonIMultiplication.Checked)
            {
                increaseFunction = new CIncreaseMultiply(Convert.ToDouble(this.numericUpDownIPheromone.Value), Convert.ToDouble(this.numericUpDownMax.Value));
            }
            else if (radioButtonIPlus.Checked)
            {
                increaseFunction = new CIncreasePlus(Convert.ToDouble(this.numericUpDownIPheromone.Value), Convert.ToDouble(this.numericUpDownMax.Value));
            }

            IPheromoneDecrease decreaseFunction = null;
            if (radioButtonDConstant.Checked)
            {
                decreaseFunction = new CDecreaseConstant(Convert.ToDouble(this.numericUpDownDPheromone.Value));
            }
            else if (radioButtonDMultiplication.Checked)
            {
                decreaseFunction = new CDecreaseMultiply(Convert.ToDouble(this.numericUpDownDPheromone.Value), Convert.ToDouble(this.numericUpDownMin.Value));
            }
            else if (radioButtonDMinus.Checked)
            {
                decreaseFunction = new CDecreaseMinus(Convert.ToDouble(this.numericUpDownDPheromone.Value), Convert.ToDouble(this.numericUpDownMin.Value));
            }

            double betterThan;
            if (this.checkBoxBetterThan.Checked)
            {
                betterThan = Convert.ToDouble(this.numericUpDownBetterThan.Value);
            }
            else
            {
                betterThan = 0;
            }

            CSettingsHolder settings = new CSettingsHolder(Convert.ToDouble(this.numericUpDownAlfa.Value), Convert.ToDouble(this.numericUpDownBeta.Value), 12, Convert.ToInt32(this.numericUpDownBestCount.Value), betterThan, increaseFunction, decreaseFunction);
            this.mainForm.ShowResults(EProblemType.TSP, settings);
        }

        private CEdge[] CalculateEdges(CNode[] nodeArray)
        {
            foreach (CNode node in nodeArray)
            {
                node.SetEdgeCount(nodeArray.Length - 1);
            }

            int counter = 0;
            int count = (nodeArray.Length * (nodeArray.Length - 1)) / 2;
            CEdge[] edgeArray = new CEdge[count];
            for (int i = 0; i < nodeArray.Length; i++)
            {
                for (int j = i + 1; j < nodeArray.Length; j++)
                {
                    edgeArray[counter] = new CEdge(counter, nodeArray[i], nodeArray[j]);
                    counter++;
                }
            }
            return edgeArray;
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.mainForm.ShowPanel(EProblemType.TSP);
        }

        private void buttonGenerate_Click(object sender, EventArgs e)
        {
            if (this.radioButtonRandom.Checked)
            {

                this.graphPanel.GraphicHolder = new CGraphicObjectHolder();
                this.graphPanel.GraphicHolder.NormalNodes = new CNodeGenerator(this.graphPanel.PanelHeight, this.graphPanel.PanelWidth).GetRandom(Convert.ToInt32(this.textBoxRandomCount.Text));
                this.graphPanel.RenderGraphic(ERenderMode.Graph);

            }
            else if (this.radioButtonSite.Checked)
            {

                this.graphPanel.GraphicHolder = new CGraphicObjectHolder();
                this.graphPanel.GraphicHolder.NormalNodes = new CNodeGenerator(this.graphPanel.PanelHeight, this.graphPanel.PanelWidth).GetSiteList(Convert.ToInt32(this.textBoxRowCount.Text), Convert.ToInt32(this.textBoxColumnCount.Text));
                this.graphPanel.RenderGraphic(ERenderMode.Graph);
                this.numericUpDownAlfa.Value = Convert.ToDecimal(3.50);
            }
            else if (this.radioButtonCircle.Checked)
            {
                this.graphPanel.GraphicHolder = new CGraphicObjectHolder();
                this.graphPanel.GraphicHolder.NormalNodes = new CNodeGenerator(this.graphPanel.PanelHeight, this.graphPanel.PanelWidth).GetCircle((this.graphPanel.Height - (this.graphPanel.Height / 7)), Convert.ToInt32(this.textBoxCircleCount.Text));
                this.graphPanel.RenderGraphic(ERenderMode.Graph);
            }
            this.buttonRun.Enabled = true;
        }

        private void ButtonClear_Click(object sender, EventArgs e)
        {
            this.graphPanel.GraphicHolder = new CGraphicObjectHolder();
            this.graphPanel.RenderGraphic(ERenderMode.None);
        }

        private void buttonChooseImportFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();
            openFileDialog1.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
            openFileDialog1.FilterIndex = 2;
            openFileDialog1.RestoreDirectory = true;
            openFileDialog1.Multiselect = false;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.textBoxImportFile.Text = openFileDialog1.FileName;
            }
        }

        private void buttonImport_Click(object sender, EventArgs e)
        {
            string[] lines = System.IO.File.ReadAllLines(this.textBoxImportFile.Text);

            List<CNode> nodeList = new List<CNode>();
            LinkedList<CEdgeLine> resultEdges = new LinkedList<CEdgeLine>();
            int counter = 0;
            bool resultNodes = false;
            foreach (string line in lines)
            {
                if (line.StartsWith("N "))
                {    
                    string[] pointArray = line.Split(' ');
                    CNodePoint point = new CNodePoint(Convert.ToInt32(pointArray[1]), Convert.ToInt32(pointArray[2]));
                    nodeList.Add(new CNode(counter, point, 0));
                    counter++;
                }
                if (line.StartsWith("NR"))
                {
                    resultNodes = true;
                    string[] pointArray = line.Split(' ');
                    CNodePoint point = new CNodePoint(Convert.ToInt32(pointArray[1]), Convert.ToInt32(pointArray[2]));
                    nodeList.Add(new CNode(counter, point, 0));
                    counter++;
                }
            }
            this.graphPanel.GraphicHolder.NormalNodes = nodeList.ToArray();
            if (resultNodes)
            {
                foreach (CNode node in nodeList)
                {
                    node.SetEdgeCount(2);
                }
                for (int i = 0; i < nodeList.Count - 1; i++)
                {
                    CEdge edge = new CEdge(i, nodeList[i], nodeList[i + 1]);
                    resultEdges.AddLast(edge.ToCEdgeLine());
                }
                CEdge edge2 = new CEdge(nodeList.Count, nodeList[nodeList.Count - 1], nodeList[0]);
                resultEdges.AddLast(edge2.ToCEdgeLine());

                this.graphPanel.GraphicHolder.ResultEdges = resultEdges;
                this.graphPanel.RenderGraphic(ERenderMode.Result);
            }
            else
            {
                this.graphPanel.GraphicHolder.ResultEdges = new LinkedList<CEdgeLine>();
                this.graphPanel.RenderGraphic(ERenderMode.All);
            }
            this.buttonRun.Enabled = true;
        }

        private void buttonChooseExportFile_Click(object sender, EventArgs e)
        {
            if (!Directory.Exists("export"))
            {
                Directory.CreateDirectory("export");
            }

            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
            saveFileDialog1.RestoreDirectory = true;
            saveFileDialog1.InitialDirectory = Directory.GetCurrentDirectory();

            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                this.textBoxExportFile.Text = saveFileDialog1.FileName;
            }
        }

        private void buttonExport_Click(object sender, EventArgs e)
        {
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(this.textBoxExportFile.Text))
            {
                foreach (CNode node in this.graphPanel.GraphicHolder.NormalNodes)
                {
                    file.WriteLine(node.ToStringExport());
                }
                MessageBox.Show("OK");
            }
        }

        private void radioButtonRandom_CheckedChanged(object sender, EventArgs e)
        {
            this.graphPanel.GraphicHolder.NormalNodes = new CNode[0];
            this.graphPanel.RenderGraphic(ERenderMode.All);
        }

        private void radioButtonSite_CheckedChanged(object sender, EventArgs e)
        {
            this.graphPanel.GraphicHolder.NormalNodes = new CNode[0];
            this.graphPanel.RenderGraphic(ERenderMode.All);
        }

        private void radioButtonCircle_CheckedChanged(object sender, EventArgs e)
        {
            this.graphPanel.GraphicHolder.NormalNodes = new CNode[0];
            this.graphPanel.RenderGraphic(ERenderMode.All);
        }
    }
}
