﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Ant_Colony_Optimalization.Enum;
using Ant_Colony_Optimalization.Generator;
using System.Threading;
using Ant_Colony_Optimalization.Graph;
using Ant_Colony_Optimalization.Ant;
using Ant_Colony_Optimalization.Graphic.Object;
using Ant_Colony_Optimalization.Other;
using System.IO;

namespace Ant_Colony_Optimalization.UserControls
{

    public partial class UserControlTSPResults : UserControl
    {
        private FormMain mainForm;
        private UserControlGraph graphPanel;
        private double bestResult;
        private ManualResetEvent _busy = new ManualResetEvent(false);
        private bool firstRun;
        private CNode[] nodeList;
        private CEdge[] edgeArray;
        private CNode selectedNode;
        private int roundCount;
        private CSettingsHolder runSettings;
        private DateTime startTime;
        private LinkedList<string> logRecordList;
        public UserControlTSPResults(CSettingsHolder runSettings)
        {
            InitializeComponent();
            this.bestResult = double.MaxValue;
            this.firstRun = true;
            this.runSettings = runSettings;
            this.logRecordList = new LinkedList<string>();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.CancelWorker();
            this.mainForm.ShowPanel(EProblemType.TSP);
        }

        private void UserControlResults_Load(object sender, EventArgs e)
        {
            this.mainForm = (FormMain)this.Parent.Parent;
            this.graphPanel = this.mainForm.GraphPanelUserControl;
        }

        private void buttonPause_Click(object sender, EventArgs e)
        {
            this.buttonPause.Enabled = false;
            this.buttonRun.Enabled = true;
            this.buttonNextRound.Enabled = true;
            this.buttonNextStep.Enabled = true;
            this.PauseWorker();
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            //this.graphPanel.GraphicHolder.AllEdges = new CEdge[0];
            this.graphPanel.RenderGraphic(ERenderMode.Result);
            this.buttonPause.Enabled = true;
            this.buttonRun.Enabled = false;
            this.buttonNextRound.Enabled = false;
            this.buttonNextStep.Enabled = false;
            this.startTime = DateTime.Now;
            //this.backgroundWorker3.RunWorkerAsync();
            this.ResumeWorker();
        }

        private void buttonNextStep_Click(object sender, EventArgs e)
        {
            this.backgroundWorker1.RunWorkerAsync();
        }

        private void backgroundWorker3_DoWork(object sender, DoWorkEventArgs e)
        {

            if (this.firstRun)
            {
                this.nodeList = this.graphPanel.GraphicHolder.NormalNodes;
                this.edgeArray = this.graphPanel.GraphicHolder.AllEdges;
            }
            int nodeListCount = this.nodeList.Count();

            List<CAntCompleteGraph> anti = new List<CAntCompleteGraph>();
            if (radioButtonAfterCount.Checked)
            {
                int iterationCount = Convert.ToInt32(this.numericUpDownIterationCount.Value) * this.graphPanel.GraphicHolder.NormalNodes.Count();
                this.roundCount = iterationCount;
            }
            if (radioButtonNoImprovement.Checked)
            {
                this.roundCount = int.MaxValue;
            }
           // int kIteration = 0;
            int noChangeFor = 0;

            for (int kIteration = 0; kIteration <= roundCount-1; kIteration++)
            {

                int nodeIndex = kIteration % nodeListCount;
                CNode startingNode = nodeList[nodeIndex];
                CAntCompleteGraph ant = new CAntCompleteGraph(kIteration,startingNode, nodeList.Count(), this.runSettings.Aplha, this.runSettings.Beta, this.runSettings.PheromonIncreaseFunction);

                while (ant.MoveToNextNode())
                {

                }
                if (anti.Count() % this.graphPanel.GraphicHolder.NormalNodes.Count() == this.graphPanel.GraphicHolder.NormalNodes.Count() - 1)
                {
                    anti.Sort();

                    for (int io = 0; io < this.runSettings.BestCount; io++)
                    {
                        if (anti.ElementAt(io).Distance < (this.bestResult * this.runSettings.BetterThan))
                        {
                            anti.ElementAt(io).ActualizePheromone();
                            this.runSettings.PheromonDecreaseFunction.DecreasePheromone(edgeArray);
                        }

                        if (anti.ElementAt(io).Distance < (this.bestResult))
                        {
                            this.graphPanel.GraphicHolder.ResultEdges = new LinkedList<CEdgeLine>();
                            for (int i = 0; i < anti.ElementAt(io).VisitedNodes.Count() - 1; i++)
                            {
                                CNode node1 = anti.ElementAt(io).VisitedNodes.ElementAt(i);
                                CNode node2 = anti.ElementAt(io).VisitedNodes.ElementAt(i).EdgeArray[anti.ElementAt(io).VisitedEdgesIndex.ElementAt(i)].GetNeighbor(anti.ElementAt(io).VisitedNodes.ElementAt(i));
                                this.graphPanel.GraphicHolder.ResultEdges.AddLast(new CEdgeLine(node1.Point, node2.Point));
                            }
                            this.bestResult = anti.ElementAt(io).Distance;
                            backgroundWorker3.ReportProgress(kIteration, new ProgressInfo(kIteration, anti.ElementAt(io).Distance));
                        }
                    }
                    anti = new List<CAntCompleteGraph>();
                }
                else
                {
                    anti.Add(ant);
                }

                
                if (ant.Distance < this.bestResult)
                {
                    
                    this.graphPanel.GraphicHolder.ResultEdges = new LinkedList<CEdgeLine>();
                    for (int i = 0; i < ant.VisitedNodes.Count() - 1; i++)
                    {
                        CNode node1 = ant.VisitedNodes.ElementAt(i);
                        CNode node2 = ant.VisitedNodes.ElementAt(i).EdgeArray[ant.VisitedEdgesIndex.ElementAt(i)].GetNeighbor(ant.VisitedNodes.ElementAt(i));
                        this.graphPanel.GraphicHolder.ResultEdges.AddLast(new CEdgeLine(node1.Point, node2.Point));
                    }
                    this.bestResult = ant.Distance;

                    backgroundWorker3.ReportProgress(kIteration, new ProgressInfo(kIteration, ant.Distance));
                    //this.runSettings.PheromonDecreaseFunction.DecreasePheromone(edgeArray);

                    //ant.ActualizePheromone();
                    noChangeFor = 0;
                    this.graphPanel.RenderGraphic(ERenderMode.Result);
                }



                /*
                if (ant.Distance < (this.bestResult  * 1.03))
                {
                    ant.ActualizePheromone();
                    this.runSettings.PheromonDecreaseFunction.DecreasePheromone(edgeArray);
                }
                */
                backgroundWorker3.ReportProgress(kIteration, null);
                _busy.WaitOne();
                if (this.backgroundWorker3.CancellationPending)
                {
                    kIteration = int.MaxValue-1;
                }

                if (this.radioButtonNoImprovement.Checked)
                {
                    noChangeFor++;
                    if (noChangeFor== (Convert.ToInt32(this.numericUpDownNoImprovement.Value) * this.graphPanel.GraphicHolder.NormalNodes.Count())){
                        kIteration = int.MaxValue-1;
                    }
                }
            }


        }

        private void backgroundWorker3_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            int nodeCount = this.graphPanel.GraphicHolder.NormalNodes.Count();
            if (e.UserState != null)
            {
                ProgressInfo pi = (ProgressInfo)e.UserState;
                this.labelBestLength.Text = pi.BestDistance.ToString();
                this.labelBestAnt.Text = pi.BestAnt.ToString();
                this.labelIterationValue.Text = (pi.BestAnt / nodeCount).ToString();
                string time = (DateTime.Now - this.startTime).ToString();
                this.labelInTimeValue.Text = time.Substring(0,time.LastIndexOf('.')+3);
                this.logRecordList.AddLast(pi.ToString() +";"+ (DateTime.Now - this.startTime).ToString());

            }
            int iteration = Convert.ToInt32(this.labelIterationActualValue.Text) + 1;
            this.labelIterationActualValue.Text = (e.ProgressPercentage/nodeCount).ToString() ;
            this.labelAntActualValue.Text = (e.ProgressPercentage).ToString();
        }

        private void buttonBackToMainMenu_Click(object sender, EventArgs e)
        {
            this.mainForm.ShowMainMenu();
        }

        public void ShowNodeInfo(CNode node)
        {
            this.selectedNode = node;
            new CNodeAnalyzator(this.dataGridView1, node, this.runSettings.Aplha, this.runSettings.Beta).ShowEdgesInfo();
            this.labelNodeInfo.Text = "Node : " + node.id.ToString();
        }

        public void ResumeWorker()
        {
            // Start the worker if it isn't running
            if (!backgroundWorker3.IsBusy) backgroundWorker3.RunWorkerAsync();
            // Unblock the worker 
            _busy.Set();
        }

        public void PauseWorker()
        {
            // Block the worker
            _busy.Reset();
        }

        public void CancelWorker()
        {
            this.PauseWorker();
            if (backgroundWorker3.IsBusy)
            {
                // Set CancellationPending property to true
                backgroundWorker3.CancelAsync();
                // Unblock worker so it can see that
                _busy.Set();
            }
        }

        private void backgroundWorker3_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            string n = string.Format("log-{0:yyyy-MM-dd_hh-mm-ss}.csv", DateTime.Now);
            if (!Directory.Exists("log"))
            {
                Directory.CreateDirectory("log");
            }
            this.logRecordList.AddFirst("distance;iteration;ant;time;nodeCount="+this.graphPanel.GraphicHolder.NormalNodes.Length);
            System.IO.File.WriteAllLines(@"log\"+n, this.logRecordList.ToArray());
            MessageBox.Show("Work finished");
            this.buttonRun.Enabled = true;
            this.buttonNextRound.Enabled = true;
            this.buttonNextStep.Enabled = true;
        }

        private void dataGridView1_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            int edgeId = Convert.ToInt32(dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[0].Value);
            this.graphPanel.GraphicHolder.PinkEdges = new LinkedList<CEdgeLine>();
            this.graphPanel.GraphicHolder.PinkEdges.AddLast(new CEdgeLine(this.selectedNode.EdgeArray[edgeId].Node1.Point, this.selectedNode.EdgeArray[edgeId].Node2.Point));
            this.graphPanel.RenderGraphic(ERenderMode.EdgeHighlighted);
        }

        private void dataGridView1_SelectionChanged(object sender, EventArgs e)
        {
           /* if (this.selectedNode != null)
            {
                int edgeId = Convert.ToInt32(dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[0].Value);
                this.graphPanel.GraphicHolder.PinkEdges = new LinkedList<CEdgeLine>();
                this.graphPanel.GraphicHolder.PinkEdges.AddLast(new CEdgeLine(this.selectedNode.EdgeArray[edgeId].Node1.Point, this.selectedNode.EdgeArray[edgeId].Node2.Point));
                this.graphPanel.RenderGraphic(ERenderMode.EdgeHighlighted);
            }*/
        }

        private void labelNodeInfo_TextChanged(object sender, EventArgs e)
        {
            this.dataGridView1.Rows[0].Selected = true;
        }

        private void buttonNextRound_Click(object sender, EventArgs e)
        {
            this.roundCount = 1;
            this.ResumeWorker();
            //this.backgroundWorker3.RunWorkerAsync();
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            if (this.firstRun)
            {
                this.nodeList = this.graphPanel.GraphicHolder.NormalNodes;
                this.edgeArray = this.graphPanel.GraphicHolder.AllEdges;
            }
            CAntCompleteGraph ant = new CAntCompleteGraph(1,nodeList[Convert.ToInt32(this.GetRandomNumber(0, this.graphPanel.GraphicHolder.NormalNodes.Length-1))], nodeList.Count(), this.runSettings.Aplha, this.runSettings.Beta, this.runSettings.PheromonIncreaseFunction);
            while (ant.MoveToNextNode())
            {

            }

            this.graphPanel.GraphicHolder.AntWay = new LinkedList<CEdgeLine>();
            for (int i = 0; i < ant.VisitedNodes.Count() - 1; i++)
            {
                CNode node1 = ant.VisitedNodes.ElementAt(i);
                CNode node2 = ant.VisitedNodes.ElementAt(i).EdgeArray[ant.VisitedEdgesIndex.ElementAt(i)].GetNeighbor(ant.VisitedNodes.ElementAt(i));
                this.graphPanel.GraphicHolder.AntWay.AddLast(new CEdgeLine(node1.Point, node2.Point));
                this.graphPanel.RenderGraphic(ERenderMode.AntWay);
                Thread.Sleep(500);
            }
        }

        private double GetRandomNumber(double minimum, double maximum)
        {
            Random random = CRandomProvider.GetThreadRandom();
            return (random.NextDouble() * (maximum - minimum) + minimum);
        }
    }
}
